@@ -23,7 +23,8 @@ extern void xs_init ();
 int
 make_embedded_interpreter(char const *progPath,
 			  char const *subFilter,
-			  int wantStatusReports)
+			  int wantStatusReports,
+			  char **env)
 {
     char *argv[6];
     int argc;
@@ -33,6 +34,9 @@ make_embedded_interpreter(char const *progPath,
 	PL_perl_destruct_level = 1;
 	perl_destruct(my_perl);
 	perl_free(my_perl);
+#ifdef PERL_SYS_TERM
+	PERL_SYS_TERM();
+#endif
 	my_perl = NULL;
 #else
 	syslog(LOG_WARNING, "Cannot destroy and recreate a Perl interpreter safely on this platform.  Filter rules will NOT be reread.");
@@ -41,16 +45,6 @@ make_embedded_interpreter(char const *progPath,
 
     }
 
-    my_perl = perl_alloc();
-    if (!my_perl) {
-	errno = ENOMEM;
-	return -1;
-    }
-#ifdef PERL_SET_CONTEXT
-    PERL_SET_CONTEXT(my_perl);
-#endif
-    PL_perl_destruct_level = 1;
-    perl_construct(my_perl);
     if (subFilter) {
 	argv[0] = "";
 	argv[1] = (char *) progPath;
@@ -74,6 +68,20 @@ make_embedded_interpreter(char const *progPath,
 	argv[3] = NULL;
 	argc = 3;
     }
+#ifdef PERL_SYS_INIT3
+    PERL_SYS_INIT3(&argc, &argv, &env);
+#endif
+
+    my_perl = perl_alloc();
+    if (!my_perl) {
+	errno = ENOMEM;
+	return -1;
+    }
+#ifdef PERL_SET_CONTEXT
+    PERL_SET_CONTEXT(my_perl);
+#endif
+    PL_perl_destruct_level = 1;
+    perl_construct(my_perl);
     perl_parse(my_perl, xs_init, argc, argv, NULL);
     perl_run(my_perl);
     return 0;
