@@ -28,9 +28,33 @@ static void DestroyHandler(EventHandler *eh);
 static void DoPendingChanges(EventSelector *es);
 
 #ifdef DEBUG_EVENT
+#ifndef EVENT_USE_POLL
 static void print_select_sets(char const *tag,
 			      int maxfdp1, fd_set *rd, fd_set *wr);
 #endif
+#endif
+
+#ifdef EVENT_USE_POLL
+static struct pollfd *Pollfds = NULL;
+static EventHandler **Fd_to_eh = NULL;
+static int Size_of_pollfds = 0;
+static int adjust_pollfds(int num) {
+    int new_size = Size_of_pollfds * 2;
+    if (new_size < num) new_size = num;
+    if (new_size < 16) new_size = 16;
+    Pollfds = realloc(Pollfds, (size_t) new_size * sizeof(struct pollfd));
+    if (!Pollfds) {
+	return -1;
+    }
+    Fd_to_eh = realloc(Fd_to_eh, (size_t) new_size * sizeof(EventHandler *));
+    if (!Fd_to_eh) {
+	return -1;
+    }
+    Size_of_pollfds = new_size;
+    return 0;
+}
+#endif
+
 
 /**********************************************************************
 * %FUNCTION: set_cloexec
@@ -95,8 +119,9 @@ Event_DestroySelector(EventSelector *es)
     DestroySelector(es);
 }
 
+#ifndef EVENT_USE_POLL
 /**********************************************************************
-* %FUNCTION: Event_HandleEvent
+* %FUNCTION: Event_HandleEventUsingSelect
 * %ARGUMENTS:
 *  es -- EventSelector
 * %RETURNS:
@@ -105,7 +130,7 @@ Event_DestroySelector(EventSelector *es)
 *  Handles a single event (uses select() to wait for an event.)
 ***********************************************************************/
 int
-Event_HandleEvent(EventSelector *es)
+Event_HandleEventUsingSelect(EventSelector *es)
 {
     fd_set readfds, writefds;
     fd_set *rd, *wr;
@@ -260,6 +285,172 @@ Event_HandleEvent(EventSelector *es)
     errno = errno_save;
     return r;
 }
+#endif
+
+#ifdef EVENT_USE_POLL
+/**********************************************************************
+* %FUNCTION: Event_HandleEventUsingPoll
+* %ARGUMENTS:
+*  es -- EventSelector
+* %RETURNS:
+*  0 if OK, non-zero on error.  errno is set appropriately.
+* %DESCRIPTION:
+*  Handles a single event (uses poll() to wait for an event.)
+***********************************************************************/
+int
+Event_HandleEventUsingPoll(EventSelector *es)
+{
+    struct timeval abs_timeout, now;
+    struct timeval timeout;
+    int tm;
+    EventHandler *eh;
+
+    int r = 0;
+    int errno_save = 0;
+    int foundTimeoutEvent = 0;
+    int pastDue;
+    int num_handlers = 0;
+    int nfds;
+    int i;
+
+    EVENT_DEBUG(("Enter Event_HandleEventUsingPoll(es=%p)\n", (void *) es));
+
+    /* Silence stupid compiler warning */
+    abs_timeout.tv_sec = 0;
+    abs_timeout.tv_usec = 0;
+
+    /* Build the pollfds array */
+    eh = es->handlers;
+    while(eh) {
+	num_handlers++;
+	eh = eh->next;
+    }
+
+    if (num_handlers > Size_of_pollfds) {
+	if (adjust_pollfds(num_handlers) < 0) {
+	    return -1;
+	}
+    }
+
+    eh = es->handlers;
+    nfds = 0;
+    for (eh=es->handlers; eh; eh=eh->next) {
+	if (eh->flags & EVENT_FLAG_DELETED) continue;
+	eh->pollflags = 0;
+	if (eh->flags & (EVENT_FLAG_READABLE|EVENT_FLAG_WRITEABLE)) {
+	    Pollfds[nfds].fd = eh->fd;
+	    Pollfds[nfds].events = 0;
+	    Fd_to_eh[nfds] = eh;
+	    if (eh->flags & EVENT_FLAG_READABLE) {
+		Pollfds[nfds].events |= POLLIN;
+	    }
+	    if (eh->flags & EVENT_FLAG_WRITEABLE) {
+		Pollfds[nfds].events |= POLLOUT;
+	    }
+	    nfds++;
+	}
+	if (eh->flags & EVENT_TIMER_BITS) {
+	    if (!foundTimeoutEvent) {
+		abs_timeout = eh->tmout;
+		foundTimeoutEvent = 1;
+	    } else {
+		if (eh->tmout.tv_sec < abs_timeout.tv_sec ||
+		    (eh->tmout.tv_sec == abs_timeout.tv_sec &&
+		     eh->tmout.tv_usec < abs_timeout.tv_usec)) {
+		    abs_timeout = eh->tmout;
+		}
+	    }
+	}
+    }
+    if (foundTimeoutEvent) {
+	gettimeofday(&now, NULL);
+
+	/* Convert absolute timeout to relative timeout for poll */
+	timeout.tv_usec = abs_timeout.tv_usec - now.tv_usec;
+	timeout.tv_sec = abs_timeout.tv_sec - now.tv_sec;
+	if (timeout.tv_usec < 0) {
+	    timeout.tv_usec += 1000000;
+	    timeout.tv_sec--;
+	}
+	if (timeout.tv_sec < 0 ||
+	    (timeout.tv_sec == 0 && timeout.tv_usec < 0)) {
+	    timeout.tv_sec = 0;
+	    timeout.tv_usec = 0;
+	}
+	tm = (timeout.tv_sec * 1000) + (timeout.tv_usec / 1000);
+    } else {
+	tm = -1;
+    }
+
+    if (nfds || foundTimeoutEvent) {
+	for(;;) {
+	    r = poll(Pollfds, nfds, tm);
+	    if (r < 0) {
+		if (errno == EINTR) continue;
+	    }
+	    break;
+	}
+    }
+
+    if (foundTimeoutEvent) {
+	gettimeofday(&now, NULL);
+    }
+
+    errno_save = errno;
+    es->nestLevel++;
+
+    if (r >= 0) {
+	/* Go through poll set and set pollflags */
+	for (i=0; i<nfds; i++) {
+	    eh = Fd_to_eh[i];
+	    if (Pollfds[i].revents & POLLIN) {
+		if (eh->flags & EVENT_FLAG_READABLE) {
+		    eh->pollflags |= EVENT_FLAG_READABLE;
+		}
+	    }
+	    if (Pollfds[i].revents & POLLOUT) {
+		if (eh->flags & EVENT_FLAG_WRITEABLE) {
+		    eh->pollflags |= EVENT_FLAG_WRITEABLE;
+		}
+	    }
+	}
+
+	/* Call handlers */
+	for (eh=es->handlers; eh; eh=eh->next) {
+	    /* Pending delete for this handler?  Ignore it */
+	    if (eh->flags & EVENT_FLAG_DELETED) continue;
+
+	    if (eh->flags & EVENT_TIMER_BITS) {
+		pastDue = (eh->tmout.tv_sec < now.tv_sec ||
+			   (eh->tmout.tv_sec == now.tv_sec &&
+			    eh->tmout.tv_usec <= now.tv_usec));
+		if (pastDue) {
+		    eh->pollflags |= EVENT_TIMER_BITS;
+		    if (eh->flags & EVENT_FLAG_TIMER) {
+			/* Timer events are only called once */
+			es->opsPending = 1;
+			eh->flags |= EVENT_FLAG_DELETED;
+		    }
+		}
+	    }
+	    /* Do callback */
+	    if (eh->pollflags) {
+		EVENT_DEBUG(("Enter callback: eh=%p flags=%u\n", eh, eh->pollflags));
+		eh->fn(es, eh->fd, eh->pollflags, eh->data);
+		EVENT_DEBUG(("Leave callback: eh=%p flags=%u\n", eh, eh->pollflags));
+	    }
+	}
+    }
+
+    es->nestLevel--;
+
+    if (!es->nestLevel && es->opsPending) {
+	DoPendingChanges(es);
+    }
+    errno = errno_save;
+    return r;
+}
+#endif
 
 /**********************************************************************
 * %FUNCTION: Event_AddHandler
@@ -628,6 +819,7 @@ Event_DebugMsg(char const *fmt, ...)
     fflush(Event_DebugFP);
 }
 
+#ifndef EVENT_USE_POLL
 static void
 print_select_sets(char const *tag, int maxfdp1, fd_set *rd, fd_set *wr)
 {
@@ -660,6 +852,7 @@ print_select_sets(char const *tag, int maxfdp1, fd_set *rd, fd_set *wr)
     fprintf(Event_DebugFP, "]\n");
 }
 #endif
+#endif
 
 /**********************************************************************
 * %FUNCTION: Event_EnableDebugging
