@@ -35,7 +35,13 @@ extern EventSelector *Event_CreateSelector(void);
 extern void Event_DestroySelector(EventSelector *es);
 
 /* Handle one event */
-extern int Event_HandleEvent(EventSelector *es);
+#ifdef EVENT_USE_POLL
+extern int Event_HandleEventUsingPoll(EventSelector *es);
+#define Event_HandleEvent Event_HandleEventUsingPoll
+#else
+extern int Event_HandleEventUsingSelect(EventSelector *es);
+#define Event_HandleEvent Event_HandleEventUsingSelect
+#endif
 
 /* Add a handler for a ready file descriptor */
 extern EventHandler *Event_AddHandler(EventSelector *es,
