@@ -2442,16 +2442,6 @@ main(int argc, char **argv)
 	}
     }
 
-    /* Write pid */
-    if (pidfile) {
-	FILE *fp = fopen(pidfile, "w");
-	if (fp) {
-	    fprintf(fp, "%d\n", (int) getpid());
-	    fclose(fp);
-	}
-	free(pidfile);
-    }
-
     (void) closelog();
     closefiles();
 
@@ -2461,6 +2451,13 @@ main(int argc, char **argv)
     open("/dev/null", O_RDWR);
 
     openlog("mimedefang", LOG_PID, facility);
+    /* Write pid */
+    if (pidfile) {
+	if (write_and_lock_pidfile(pidfile) < 0) {
+	    exit(1);
+	}
+	free(pidfile);
+    }
 
     if (ValidateHeader[0]) {
 	syslog(LOG_DEBUG, "IP validation header is %s", ValidateHeader);
