@@ -38,6 +38,7 @@ static char const RCSID[] =
 #include <errno.h>
 #include <stdarg.h>
 #include <netinet/in.h>
+#include <fcntl.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -1008,6 +1009,8 @@ remove_local_socket(char const *str)
 *                   local:/path/to/sock
 *                   inet:port  (host defaults to LOOPBACK)
 *                   inet_any:port (host defaults to INADDR_ANY)
+*                   inet6:port (host defaults to in6addr_loopback)
+*                   inet6_any:port (host defaults to in6addr_any)
 *  backlog -- listen backlog.  If -1, we use default of 5
 *  must_be_unix -- If true, do not accept the inet: or inet_any: sockets.
 * %RETURNS:
@@ -1072,6 +1075,55 @@ make_listening_socket(char const *str, int backlog, int must_be_unix)
 	    close(sock);
 	    return -1;
 	}
+    } else if (!strncmp(str, "inet6:", 6) ||
+	       !strncmp(str, "inet6_any:", 10)) {
+#if defined(AF_INET6)
+	struct in6_addr const *bind6_addr;
+	struct sockaddr_in6 addr6;
+	if (must_be_unix) {
+	    return -2;
+	}
+	if (!strncmp(str, "inet6:", 6)) {
+	    path = str+6;
+	    bind6_addr = &in6addr_loopback;
+	} else {
+	    path = str+10;
+	    bind6_addr = &in6addr_any;
+	}
+	if (sscanf(path, "%d", &port) != 1 ||
+	    port < 1 ||
+	    port > 65535) {
+	    syslog(LOG_ERR, "make_listening_socket: Invalid port %s", path);
+	    errno = EINVAL;
+	    return -1;
+	}
+
+	sock = socket(PF_INET6, SOCK_STREAM, 0);
+	if (sock < 0) {
+	    syslog(LOG_ERR, "make_listening_socket: socket: %m");
+	    return -1;
+	}
+
+	opt = 1;
+	/* Reuse port */
+	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)) < 0) {
+	    syslog(LOG_ERR, "make_listening_socket: setsockopt: %m");
+	    close(sock);
+	    return -1;
+	}
+	addr6.sin6_family = AF_INET6;
+	addr6.sin6_port   = htons(port);
+	addr6.sin6_addr   = *bind6_addr;
+	if (bind(sock, (struct sockaddr *) &addr6, sizeof(addr6)) < 0) {
+	    syslog(LOG_ERR, "make_listening_socket: bind: %m");
+	    close(sock);
+	    return -1;
+	}
+#else
+	syslog(LOG_ERR, "Cannot specify inet6 socket: No IPv6 support");
+	fprintf(stderr, "Cannot specify inet6 socket: No IPv6 support\n");
+	return -1;
+#endif
     } else {
 	/* Assume unix-domain socket */
 	path = str;
@@ -1119,6 +1171,7 @@ make_listening_socket(char const *str, int backlog, int must_be_unix)
 *                   unix:/path/to/sock
 *                   local:/path/to/sock
 *                   inet:port  (host defaults to LOOPBACK)
+*                   inet6:port (host defaults to in6addr_loopback)
 * %RETURNS:
 *  Connects on specified UNIX-domain or TCP socket.
 * %DESCRIPTION:
@@ -1157,6 +1210,37 @@ connect_to_socket(char const *str)
 	    close(sock);
 	    return -1;
 	}
+    } else if (!strncmp(str, "inet6:", 6)) {
+#if defined(AF_INET6)
+	struct sockaddr_in6 addr_in6;
+	path = str+5;
+	if (sscanf(path, "%d", &port) != 1 ||
+	    port < 1 ||
+	    port > 65535) {
+	    syslog(LOG_ERR, "connect_to_socket: Invalid port %s", path);
+	    errno = EINVAL;
+	    return -1;
+	}
+
+	sock = socket(PF_INET6, SOCK_STREAM, 0);
+	if (sock < 0) {
+	    syslog(LOG_ERR, "connect_to_socket: socket: %m");
+	    return -1;
+	}
+
+	addr_in6.sin6_family = AF_INET6;
+	addr_in6.sin6_port = htons(port);
+	addr_in6.sin6_addr = in6addr_loopback;
+	if (connect(sock, (struct sockaddr *) &addr_in6, sizeof(addr_in6)) < 0) {
+	    syslog(LOG_ERR, "connect_to_socket: connect: %m");
+	    close(sock);
+	    return -1;
+	}
+#else
+	syslog(LOG_ERR, "Cannot specify inet6 socket: No IPv6 support");
+	fprintf(stderr, "Cannot specify inet6 socket: No IPv6 support\n");
+	return -1;
+#endif
     } else {
 	/* Assume unix-domain socket */
 	path = str;
@@ -1253,3 +1337,32 @@ free_debug(void *ctx, void *x, char const *fname, int line)
     (free)(x);
 }
 #endif
+
+int
+write_and_lock_pidfile(char const *pidfile)
+{
+    int fd;
+    struct flock fl;
+    char buf[64];
+
+    fl.l_type = F_WRLCK;
+    fl.l_whence = SEEK_SET;
+    fl.l_start = 0;
+    fl.l_len = 0;
+
+    fd = open(pidfile, O_RDWR|O_CREAT, 0666);
+    if (fd < 0) {
+	syslog(LOG_ERR, "Could not open PID file %s: %m", pidfile);
+	return -1;
+    }
+    if (fcntl(fd, F_SETLK, &fl) < 0) {
+	syslog(LOG_ERR, "Could not lock PID file %s: %m.  Is another copy running?", pidfile);
+	return -1;
+    }
+    ftruncate(fd, 0);
+    snprintf(buf, sizeof(buf), "%lu\n", (unsigned long) getpid());
+    write(fd, buf, strlen(buf));
+    /* Do NOT close fd... it will close and lock will be released
+       when we exit */
+    return 0;
+}
